@@ -2,14 +2,15 @@
 #### (called from more than one place --> need to be defined early)
 
 ## Need to consider NAs ;  "== 0" even works for logical & complex:
+## Note that "!x" is faster than "x == 0", but does not (yet!) work for complex
 is0  <- function(x) !is.na(x) & x == 0
 isN0 <- function(x)  is.na(x) | x != 0
-all0 <- function(x) !any(is.na(x)) && all(x == 0)
-
-## These work "identically" for  1 ('==' TRUE)  and 0 ('==' FALSE):
+all0 <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
+## These work "identically" for  1 ('==' TRUE)  and 0 ('==' FALSE)
+##	(but give a warning for "double"  1 or 0)
 ## TODO: C versions of these would be faster
 allTrue  <- function(x) all(x)  && !any(is.na(x))
-allFalse <- function(x) !any(x) && !any(is.na(x))
+allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0
 
 as1 <- function(x, mod=mode(x))
     switch(mod, "integer" = 1L, "numeric" = 1, "logical" = TRUE, "complex" = 1+0i)
@@ -28,6 +29,10 @@ paste0 <- function(...) paste(..., sep = '')
 
 .if.NULL <- function(x, orElse) if(!is.null(x)) x else orElse
 
+##  not %in%  :
+`%nin%` <- function (x, table) is.na(match(x, table))
+
+
 .has.DN <- ## has non-trivial Dimnames slot?
     function(x) !identical(list(NULL,NULL), x@Dimnames)
 
@@ -93,21 +98,23 @@ MatrixClass <- function(cl, cld = getClassDef(cl),
     }
 }
 
-attrSlotNames <- function(m) {
+attrSlotNames <- function(m, factors = TRUE) {
     ## slotnames of Matrix objects which are *not* directly content related
-    sn <- slotNames(m); sn[!(sn %in% c("x","i","j","p"))]
+    sn <- slotNames(m); sn[sn %nin% c("x","i","j","p", if(!factors) "factors")]
 }
 
 ##' @param m
 ##' @return the slots of 'm' which are "attributes" of some kind.
-attrSlots <- function(m) sapply(attrSlotNames(m), function(sn) slot(m, sn),
-				simplify = FALSE)
+attrSlots <- function(m, factors = TRUE) sapply(attrSlotNames(m, factors=factors),
+			 function(sn) slot(m, sn), simplify = FALSE)
 
-attr.all_Mat <- function(target, current, check.attributes = TRUE, ...) {
+attr.all_Mat <- function(target, current,
+			 check.attributes = TRUE, factorsCheck = FALSE, ...) {
     msg <- if(check.attributes)
-	attr.all.equal(attrSlots(target), attrSlots(current), ...)
+	all.equal(attrSlots(target,  factors=factorsCheck),
+		  attrSlots(current, factors=factorsCheck),
+		  check.attributes = TRUE, ...)
     if((c1 <- class(target)) != (c2 <- class(current)))
-
 	## list(): so we can easily check for this
 	list(c(msg, paste("class(target) is ", class(target), ", current is ",
 				 class(current), sep = "")))
@@ -1092,9 +1099,15 @@ isTriC <- function(object, upper = NA) {
     if(d[1] != d[2]) return(FALSE)
     ## else
     TRUE.U <- structure(TRUE, kind = "U")
-    if(d[1] == 0) return(TRUE.U)
+    if((n <- d[1]) == 0) return(TRUE.U)
     TRUE.L <- structure(TRUE, kind = "L")
-    ni <- 1:d[2]
+    ## Need this, since 'i' slot of symmetric looks like triangular :
+    if(is(object, "symmetricMatrix")) # triangular only iff diagonal :
+        return(if(length(oi <- object@i) == n && identical(oi, 0:(n-1L))
+                  && identical(object@p, 0:n))
+               structure(TRUE, kind = object@uplo) else FALSE)
+    ## else
+    ni <- 1:n
     ## the row indices split according to column:
     ilist <- split(object@i, factor(rep.int(ni, diff(object@p)), levels= ni))
     lil <- unlist(lapply(ilist, length), use.names = FALSE)
@@ -1275,3 +1288,31 @@ setZero <- function(x) {
     else ## save memory (for large sparse M):
 	as.vector(x[1,1])[FALSE]
 }
+
+##' Compute the three "parts" of two sets:
+##' @param x  arbitrary vector; possibly with duplicated values,
+##' @param y  (ditto)
+##'
+##' @return list(x.only = setdiff(x,y),
+##'              y.only = setdiff(y,x), int = intersect(x,y))
+setparts <- function(x,y, uniqueCheck = TRUE, check = TRUE) {
+    if(check) {
+        x <- as.vector(x)
+        y <- as.vector(y)
+    }
+    if(uniqueCheck) {
+        x <- unique.default(x)
+        y <- unique.default(y)
+    }
+    .setparts(x,y)
+}
+.setparts <- function(x,y) {
+    n1 <- length(m1 <- match(x,y, 0L))
+    n2 <- length(m2 <- match(y,x, 0L))
+    ix <- seq_len(n1)[m1 == 0L]
+    iy <- seq_len(n2)[m2 == 0L]
+    list(x.only = x[ix], ix.only = ix, mx = m1,
+         y.only = y[iy], iy.only = iy, my = m2,
+         int = if(n1 < n2) y[m1] else x[m2])
+}
+
