@@ -3,8 +3,6 @@
 #include "Mutils.h"
 #include <R_ext/Lapack.h>
 
-#include <Rversion.h>
-
 #if 0 /* defined(R_VERSION) && R_VERSION >= R_Version(2, 7, 0) *
        * La_norm_type() & La_rcond_type() are now in R_ext/Lapack.h
        * but because of the 'module-mess' that's not sufficient */
@@ -94,8 +92,8 @@ set_double_by_name(SEXP obj, double val, char *nm)
 SEXP as_det_obj(double val, int log, int sign)
 {
     SEXP det = PROTECT(allocVector(VECSXP, 2)),
-	nms = allocVector(STRSXP, 2),
-	vv = ScalarReal(val);
+	nms = PROTECT(allocVector(STRSXP, 2)),
+	vv = PROTECT(ScalarReal(val));
 
     setAttrib(det, R_NamesSymbol, nms);
     SET_STRING_ELT(nms, 0, mkChar("modulus"));
@@ -104,7 +102,7 @@ SEXP as_det_obj(double val, int log, int sign)
     SET_VECTOR_ELT(det, 0, vv);
     SET_VECTOR_ELT(det, 1, ScalarInteger(sign));
     setAttrib(det, R_ClassSymbol, mkString("det"));
-    UNPROTECT(1);
+    UNPROTECT(3);
     return det;
 }
 
@@ -136,6 +134,7 @@ SEXP set_factors(SEXP obj, SEXP val, char *nm)
 
     if ((!isNewList(fac)) || (length(fac) > 0 && nms == R_NilValue))
 	error(_("'factors' slot must be a named list"));
+    PROTECT(val); /* set_factors(..) may be called as "finalizer" after UNPROTECT()*/
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
 	    SET_VECTOR_ELT(fac, i, duplicate(val));
@@ -152,13 +151,13 @@ SEXP set_factors(SEXP obj, SEXP val, char *nm)
     SET_VECTOR_ELT(nfac, len, duplicate(val));
     SET_STRING_ELT(nnms, len, mkChar(nm));
     SET_SLOT(obj, Matrix_factorSym, nfac);
-    UNPROTECT(2);
+    UNPROTECT(3);
     return VECTOR_ELT(nfac, len);
 }
 
 #if 0 				/* unused */
 /* useful for all the ..CMatrix classes (and ..R by [0] <-> [1]); but unused */
-SEXP dgCMatrix_set_Dim(SEXP x, int nrow)
+SEXP CMatrix_set_Dim(SEXP x, int nrow)
 {
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
 
@@ -222,6 +221,7 @@ void make_i_matrix_symmetric(int *to, SEXP from)
     MAKE_SYMMETRIC_BODY(to, from)
 
 
+#if R_VERSION < R_Version(2, 10, 0)
 /**
  * Create a named vector of type TYP
  *
@@ -244,9 +244,21 @@ Matrix_make_named(int TYP, const char **names)
     UNPROTECT(2);
     return ans;
 }
+#endif
+
 
 #define Matrix_Error_Bufsiz    4096
 
+/**
+ * Check validity of 1-letter string from a set of possible values
+ * (typically used in  S4 validity method)
+ *
+ * @param sP
+ * @param vals a string containing the possible valid letters
+ * @param nm   the name of the slot being checked
+ *
+ * @return a SEXP, either NULL (= success) or an error message
+ */
 SEXP check_scalar_string(SEXP sP, char *vals, char *nm)
 {
     SEXP val = ScalarLogical(1);
@@ -740,8 +752,7 @@ SEXP new_dgeMatrix(int nrow, int ncol)
 }
 
 /**
- * Return the 0-based index of a string match in a vector of strings
- * terminated by an empty string.  Returns -1 for no match.
+ * Encode Matrix index (i,j)  |-->  i + j * nrow   {i,j : 0-origin}
  *
  * @param ij: 2-column integer matrix
  * @param di: dim(.), i.e. length 2 integer vector
@@ -780,8 +791,7 @@ SEXP m_encodeInd(SEXP ij, SEXP di)
 #undef do_ii_FILL
 
 /**
- * Return the 0-based index of a string match in a vector of strings
- * terminated by an empty string.  Returns -1 for no match.
+ * Encode Matrix index (i,j)  |-->  i + j * nrow   {i,j : 0-origin}
  *
  * @param i: integer vector
  * @param j: integer vector of same length as 'i'
