@@ -17,6 +17,8 @@ extern "C" {
 #define _(String) dgettext ("Matrix", String)
 #else
 #define _(String) (String)
+/* Note that this is not yet supported (for Windows, e.g.) in R 2.9.0 : */
+#define dngettext(pkg, String, StringP, N) (N > 1 ? StringP : String)
 #endif
 
 #ifdef __GNUC__
@@ -69,7 +71,12 @@ SEXP dgCMatrix_set_Dim(SEXP x, int nrow);
 /* int csc_unsorted_columns(int ncol, const int p[], const int i[]); */
 /* void csc_sort_columns(int ncol, const int p[], int i[], double x[]); */
 /* SEXP csc_check_column_sorting(SEXP A); */
+#if R_VERSION < R_Version(2, 10, 0)
 SEXP Matrix_make_named(int TYP, const char **names);
+#else
+# define Matrix_make_named Rf_mkNamed
+#endif
+
 SEXP check_scalar_string(SEXP sP, char *vals, char *nm);
 Rboolean equal_string_vectors(SEXP s1, SEXP s2);
 
@@ -212,7 +219,7 @@ int* expand_cmprPt(int ncol, const int mp[], int mj[])
  * @return Rboolean :== any(is.na(slot(obj, "x") )
  */
 static R_INLINE
-Rboolean any_NA(SEXP obj)
+Rboolean any_NA_in_x(SEXP obj)
 {
     double *x = REAL(GET_SLOT(obj, Matrix_xSym));
     int i, n = LENGTH(GET_SLOT(obj, Matrix_xSym));
