@@ -39,7 +39,7 @@ SEXP dgeMatrix_validate(SEXP obj)
 static
 double get_norm(SEXP obj, const char *typstr)
 {
-    if(any_NA(obj))
+    if(any_NA_in_x(obj))
 	return NA_REAL;
     else {
 	char typnm[] = {'\0', '\0'};
