@@ -67,8 +67,8 @@ all.equal((Atr %*% solve(Atr, y))@x, y)
 ## solve() for dtC*
 mc <- round(chol(crossprod(A)), 2)
 B <- A[1:3,] # non-square on purpose
-stopifnot(all.equal(sum(rowSums(B %*% mc)), 5.82424475145),
-	  identical(tcrossprod(B, mc), t(tcrossprod(mc, B))))
+stopifnot(all.equal(sum(rowSums(B %*% mc)), 5.82424475145))
+assert.EQ.mat(tcrossprod(B, mc), as.matrix(t(tcrossprod(mc, B))))
 
 m <- kronecker(Diagonal(2), mc)
 stopifnot(is(mc, "Cholesky"),
@@ -88,12 +88,47 @@ stopifnot(is(m, "triangularMatrix"), is(m, "sparseMatrix"),
           is(im, "dtCMatrix"), is(itm, "dtCMatrix"), is(iitm, "dtCMatrix"),
 	  del < 1e-15)
 
+t.crossprod <- function(x, y=NULL)
+    if(getRversion() < "2.9.1" && R.version$`svn rev` < 48575)
+    ## tcrossprod() bug in "base R"
+    x %*% t(y) else tcrossprod(x, y)
+## crossprod(.,.) & tcrossprod(),  mixing dense & sparse
+v <- c(0,0,2:0)
+(V <- Matrix(v, 5,1, sparse=TRUE))
+sv <- as(v, "sparseVector")
+a <- as.matrix(A)
+cav <-  crossprod(a,v)
+tva <- t.crossprod(v,a)
+assert.EQ.mat(crossprod(A, V), cav) # gave infinite recursion
+assert.EQ.mat(crossprod(A,sv), cav)
+assert.EQ.mat(tcrossprod( sv, A), tva)
+assert.EQ.mat(tcrossprod(t(V),A), tva)
+
+M <- Matrix(0:5, 2,3) ; sM <- as(M, "sparseMatrix"); m <- as(M, "matrix")
+v <- 1:3; v2 <- 2:1
+sv  <- as( v, "sparseVector")
+sv2 <- as(v2, "sparseVector")
+tvm <- t.crossprod(v, m)
+assert.EQ.mat(tcrossprod( v, M), tvm)
+assert.EQ.mat(tcrossprod( v,sM), tvm)
+assert.EQ.mat(tcrossprod(sv,sM), tvm)
+assert.EQ.mat(tcrossprod(sv, M), tvm)
+assert.EQ.mat(crossprod(M, sv2), crossprod(m, v2))
+stopifnot(identical(tcrossprod(v, M), v %*% t(M)),
+	  identical(tcrossprod(v,sM), v %*% t(sM)),
+	  identical(tcrossprod(v, M), crossprod(v, t(M))),
+	  identical(tcrossprod(sv,sM), sv %*% t(sM)),
+	  identical(crossprod(sM, sv2), t(sM) %*% sv2),
+	  identical(crossprod(M, v2), t(M) %*% v2))
+
 ## *unit* triangular :
 t1 <- new("dtTMatrix", x= c(3,7), i= 0:1, j=3:2, Dim= as.integer(c(4,4)))
 ## from  0-diagonal to unit-diagonal {low-level step}:
 tu <- t1 ; tu@diag <- "U"
 cu <- as(tu, "dtCMatrix")
 cl <- t(cu) # unit lower-triangular
+cl10 <- cl %*% Diagonal(4, x=10)
+assert.EQ.mat(cl10, as(cl, "matrix") %*% diag(4, x=10))
 stopifnot(is(cl,"dtCMatrix"), cl@diag == "U")
 (cu2 <- cu %*% cu)
 cl2 <- cl %*% cl
@@ -101,8 +136,10 @@ validObject(cl2)
 cl2
 cu2. <- Diagonal(4) + Matrix(c(rep(0,9),14,0,0,6,0,0,0), 4,4)
 stopifnot(all(cu2 == cu2.),# was wrong for ver. <= 0.999375-4
-          is(cu2, "dtCMatrix"), is(cl2, "dtCMatrix"), # triangularity preserved
-          cu2@diag == "U", cl2@diag == "U",# UNIT-triangularity preserved
+	  is(cu2, "dtCMatrix"), is(cl2, "dtCMatrix"), # triangularity preserved
+	  cu2@diag == "U", cl2@diag == "U",# UNIT-triangularity preserved
+	  all.equal(Diagonal(4, x=10) %*% cu,
+		    Diagonal(4, x=10) %*% as.matrix(cu)),
           identical(t(cl2), cu2), # !!
           identical( crossprod(cu), Matrix( crossprod(as.matrix(cu)),sparse=TRUE)),
           identical(tcrossprod(cu), Matrix(tcrossprod(as.matrix(cu)),sparse=TRUE)))
